@@ -450,7 +450,8 @@ make_stancode <- function(PSobject, filename = NULL, debug = FALSE) {
     "generated quantities {",
     paste0("    vector[", SZDG_max['S'] + 1, "] strata_prob; // the probability of being in each stratum"),
     ifelse(Y_type == "survival",
-           paste0("    matrix[", SZDG_max['G'], ", T] mean_surv_prob; // mean survival probability"),
+           paste0("    matrix[", SZDG_max['G'], ", T] mean_surv_prob; // mean survival probability\n",
+                  "    matrix[", SZDG_max['G'], ", T] mean_RACE; // mean restricted average causal effect"),
            paste0("    vector[", SZDG_max['G'], "] mean_effect; // mean response")),
     "    {",
     paste0("        matrix[N, ", SZDG_max['S'] + 1, "] log_prob;")
@@ -458,8 +459,10 @@ make_stancode <- function(PSobject, filename = NULL, debug = FALSE) {
   if(Y_type == "survival"){
     generated_quantities_output_lines <- c(
       generated_quantities_output_lines,
-      paste0("        matrix[", SZDG_max['G'], ", T] numer;"),
+      paste0("        matrix[", SZDG_max['G'], ", T] numer_surv_prob;"),
+      paste0("        matrix[", SZDG_max['G'], ", T] numer_RACE;"),
       paste0("        matrix[N, T] expected_surv_prob[", SZDG_max['G'], "];"),
+      paste0("        matrix[N, T] expected_RACE[", SZDG_max['G'], "];"),
       "        for (i in 1:N)",
       paste0("            for (j in 1:", SZDG_max['G'], ")"),
       "                for (t in 1:T) {",
@@ -467,12 +470,24 @@ make_stancode <- function(PSobject, filename = NULL, debug = FALSE) {
              ifelse(Y_re == 0, "", 
                     paste(sapply(1:Y_re, function(i) paste0(" + XG_RE_", i, "[i] * beta_G_RE_", i, "[j]'")))),
              ");"
-      ),
-      ifelse(Y_family == "survival_Cox", 
-             "                    expected_surv_prob[j][i,t] = exp(-exp(mu) * pow(time[t], exp(theta[j])));", 
-             ifelse(Y_family == "survival_AFT", 
-                    "                    expected_surv_prob[j][i,t] = 1 - normal_cdf(log(time[t]), mu, sigma[j]);", "")),
-      "                }"
+      )
+    )
+    if (Y_family == "survival_Cox") {
+      generated_quantities_output_lines <- c(
+        generated_quantities_output_lines,
+        "                    expected_surv_prob[j][i,t] = exp(-exp(mu) * pow(time[t], exp(theta[j])));",
+        "                    expected_RACE[j][i,t] = exp(-theta[j] - mu * exp(-theta[j])) * tgamma(exp(-theta[j])) * gamma_p(exp(-theta[j]), exp(mu) * pow(time[t], exp(theta[j])));"
+      )
+    }
+    else if (Y_family == "survival_AFT") {
+      generated_quantities_output_lines <- c(
+        generated_quantities_output_lines,
+        "                    expected_surv_prob[j][i,t] = 1 - normal_cdf(log(time[t]), mu, sigma[j]);", 
+        "                    expected_RACE[j][i,t] = time[t] - normal_cdf(log(time[t]), mu, sigma[j]) + exp(mu + sigma[j] * sigma[j] / 2) * normal_cdf(log(time[t]), mu + sigma[j] * sigma[j], sigma[j]);"
+      )
+    }
+    generated_quantities_output_lines <- c(
+      generated_quantities_output_lines, "                }"
     )
   } else {
     generated_quantities_output_lines <- c(
@@ -506,8 +521,10 @@ make_stancode <- function(PSobject, filename = NULL, debug = FALSE) {
       generated_quantities_output_lines,
       paste0("        for (g in 1:", SZDG_max['G'], ") {"),
       "            for (t in 1:T) {",
-      "                numer[g, t] = mean(expected_surv_prob[g][:, t] .* exp(log_prob[:, S[g]]));",
-      "                mean_surv_prob[g, t] = numer[g, t] / strata_prob[S[g]];",
+      "                numer_surv_prob[g, t] = mean(expected_surv_prob[g][:, t] .* exp(log_prob[:, S[g]]));",
+      "                numer_RACE[g, t] = mean(expected_RACE[g][:, t] .* exp(log_prob[:, S[g]]));",
+      "                mean_surv_prob[g, t] = numer_surv_prob[g, t] / strata_prob[S[g]];",
+      "                mean_RACE[g, t] = numer_RACE[g, t] / strata_prob[S[g]];",     
       "            }",
       "        }"
     )
@@ -527,7 +544,7 @@ make_stancode <- function(PSobject, filename = NULL, debug = FALSE) {
   )
   
   comment_lines <- c(
-    "// Automatically generated by PStrata 0.0.1",
+    "// Automatically generated by PStrata 0.0.4",
     paste0("// at ", Sys.time(), " ", Sys.timezone())
   )
   
